@@ -1,7 +1,7 @@
 from collections import Counter
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional
 
-from pydantic import BaseModel, validator
+from pydantic import BaseModel, validator, conlist
 from sqlalchemy import MetaData, Table
 
 from ..core.errors import DBError, Exc, Ok
@@ -19,21 +19,25 @@ class DDL(BaseModel):
         unique: Optional[bool] = False
 
     class Index(BaseModel):
-        columns: List[str]
+        columns: conlist(str, min_items=1)
 
-    columns: Optional[List[Union[str, Column]]] = list()
+    columns: Optional[List[Column]] = list()
     indexes: Optional[Dict[str, Index]] = dict()
+    primary_key: Optional[
+        List[str]
+    ] = []  # logic field - i.e. not added by the user in the ddl definition
     permissions: Optional[Dict[str, str]] = dict()
 
+    @validator("columns", pre=True)
+    def transform_str_cols(cls, v, values):
+        if v is not None and isinstance(v, List):
+            return [{"name": c} if isinstance(c, str) else c for c in v]
+        else:
+            return v
+
     @validator("columns")
     def columns_unique(cls, v, values):
-        dupes = {
-            k
-            for k, v in Counter(
-                [e.name if isinstance(e, cls.Column) else e for e in v]
-            ).items()
-            if v > 1
-        }
+        dupes = {k for k, v in Counter([e.name for e in v]).items() if v > 1}
         if len(dupes) > 0:
             raise ValueError(f"Duplicate columns: {','.join(dupes)}")
         else:
@@ -41,7 +45,7 @@ class DDL(BaseModel):
 
     @validator("indexes")
     def index_columns_exists(cls, v, values):
-        cols = [c for i in v.values() for c in i.columns]
+        cols = [c.name for c in values.get("columns", list())]
         if len(cols) > 0:
             missing_cols = group_list(
                 [
@@ -59,13 +63,34 @@ class DDL(BaseModel):
 
         return v
 
+    @validator("primary_key", always=True)
+    def set_pk(cls, v, values):
+        columns_pk = [c.name for c in values.get("columns", []) if c.primary]
+
+        indexes_pk = list()
+        if values.get("indexes", {}).get("primary_key") is not None:
+            indexes_pk = values.get("indexes").get("primary_key").columns
+
+        if len(columns_pk) > 0 and len(indexes_pk) > 0:
+            if set(columns_pk) != set(indexes_pk):
+                columns_pk_str = " ,".join(columns_pk)
+                indexes_pk_str = " ,".join(indexes_pk)
+                raise ValueError(
+                    f"Primary key defined in indexes ({indexes_pk_str}) does not match primary key defined in columns ({columns_pk_str})."
+                )
+
+        pk = columns_pk if len(columns_pk) > 0 else indexes_pk
+
+        return pk
+
     def get_ddl(self):
         return {
-            "columns": [
-                {"name": c} if isinstance(c, str) else c.dict() for c in self.columns
-            ],
-            "indexes": {k: v.dict() for k, v in self.indexes.items()},
+            "columns": [c.dict() for c in self.columns],
+            "indexes": {
+                k: v.dict() for k, v in self.indexes.items() if k != "primary_key"
+            },
             "permissions": self.permissions,
+            "primary_key": self.primary_key,
         }
 
 
@@ -92,6 +117,7 @@ class Database:
     #   - INSERT TABLE NO PARENTHESES
     #   - DROP CASCADE
     #   - NO SET SCHEMA
+    #   - NO ALTER INDEXES
 
     def __init__(self, name, name_in_settings, db_type, common_params):
         self.name = name
@@ -138,7 +164,7 @@ class Database:
         with self.engine.connect().execution_options(autocommit=True) as connection:
             connection.execute(script)
 
-    def select(self, query, **params):
+    def read_data(self, query, **params):
         """Executes the query and returns a list of dictionaries with the data.
 
         Args:
@@ -157,10 +183,10 @@ class Database:
 
         return [dict(zip(res.keys(), r)) for r in res.fetchall()]
 
-    def select_stream(self, query, **params):
+    def _read_data_stream(self, query, **params):
         """Executes the query and returns an iterator dictionaries with the data.
 
-        The main difference with select() is that this method executes the query with a server-side
+        The main difference with read_data() is that this method executes the query with a server-side
         cursor (sqlalchemy stream_results = True).
 
         Args:
@@ -189,7 +215,7 @@ class Database:
             data (list): A list of dictionaries to load
             schema (str): An optional schema to reference the table
         """
-        table_def = self.get_table(table, schema)
+        table_def = self._get_table(table, schema)
         if table_def is None:
             raise DBError(
                 self.name,
@@ -222,7 +248,7 @@ class Database:
         batch_size = batch_size or self.max_batch_rows
         buffer = list()
         if replace:
-            self.drop_table(table, schema, execute=True)
+            self._drop_table(table, schema, execute=True)
 
         result = self._validate_ddl(ddl)
         if result.is_err:
@@ -234,10 +260,12 @@ class Database:
             )
         else:
             ddl = result.value
+
         check_create = True
+        table_exists_prior_load = self._table_exists(table, schema)
 
         for i, record in enumerate(data):
-            if check_create and not self.table_exists(table, schema):
+            if check_create and not table_exists_prior_load:
                 # Create the table if required
                 if len(ddl.get("columns", list())) == 0:
                     # If no columns are specified in the ddl, figure that out
@@ -248,7 +276,7 @@ class Database:
                     ]
                     ddl = dict(ddl, columns=columns)
 
-                self.create_table_ddl(table, schema, ddl, execute=True)
+                self._create_table_ddl(table, schema, ddl, execute=True)
                 check_create = False
 
             if i % batch_size == 0 and len(buffer) > 0:
@@ -260,7 +288,7 @@ class Database:
         if len(buffer) > 0:
             self._load_data_batch(table, buffer, schema)
 
-    def get_table(self, table, schema):
+    def _get_table(self, table, schema):
         """Create a SQLAlchemy Table object.
 
         Args:
@@ -278,8 +306,8 @@ class Database:
             )
             return table_def
 
-    def table_exists(self, table, schema):
-        return self.get_table(table, schema) is not None
+    def _table_exists(self, table, schema):
+        return self._get_table(table, schema) is not None
 
     # ETL steps
     # =========
@@ -287,10 +315,10 @@ class Database:
     # copy tasks. Can optionally also execute the
     # sql if `execute=True`
 
-    def create_table_select(
+    def _create_table_select(
         self, table, schema, select, view=False, ddl=dict(), execute=False
     ):
-        """Returns SQL code for a create table from a select statment.
+        """Returns SQL code for a create table from a select statement.
 
         Args:
             table (str): The target table name
@@ -321,8 +349,8 @@ class Database:
 
         return q
 
-    def create_table_ddl(self, table, schema, ddl, execute=False):
-        """Returns SQL code for a create table from a select statment.
+    def _create_table_ddl(self, table, schema, ddl, execute=False):
+        """Returns SQL code for a create table from a select statement.
 
         Args:
             table (str): The target table name
@@ -358,18 +386,25 @@ class Database:
             ]
         )
 
+        if len(ddl["primary_key"]) > 0:
+            # we pop the primary key to ensure it is not used again in the create_indexes step
+            pk = " ,".join(ddl["primary_key"])
+            pk = f"    , PRIMARY KEY ({pk})"
+        else:
+            pk = ""
+
         q = ""
         if_not_exists = (
             " IF NOT EXISTS" if "CREATE IF NOT EXISTS" in self.sql_features else ""
         )
-        q += f"CREATE TABLE{if_not_exists} {table} (\n      {columns}\n);"
+        q += f"CREATE TABLE{if_not_exists} {table} (\n      {columns}\n{pk}\n);"
 
         if execute:
             self.execute(q)
 
         return q
 
-    def create_indexes(self, table, schema, ddl, execute=False):
+    def _create_indexes(self, table, schema, ddl, execute=False):
         """Returns SQL to create indexes from ddl.
 
         Args:
@@ -387,12 +422,11 @@ class Database:
         indexes = {
             idx: idx_def["columns"]
             for idx, idx_def in ddl.get("indexes", dict()).items()
-            if idx != "primary_key"
         }
 
         q = ""
-        if "primary_key" in ddl.get("indexes", dict()):
-            pk_cols = ", ".join(ddl["indexes"]["primary_key"]["columns"])
+        if len(ddl["primary_key"]) > 0:
+            pk_cols = ", ".join(ddl["primary_key"])
             q += f"ALTER TABLE {table} ADD PRIMARY KEY ({pk_cols});"
 
         q += "\n".join(
@@ -408,7 +442,7 @@ class Database:
         return q
 
     def grant_permissions(self, table, schema, ddl, execute=False):
-        """Returns a set of GRANT statments.
+        """Returns a set of GRANT statements.
 
         Args:
             table (str): The target table name
@@ -431,7 +465,7 @@ class Database:
 
         return q
 
-    def drop_table(self, table, schema, view=False, execute=False):
+    def _drop_table(self, table, schema, view=False, execute=False):
         """Returns a DROP statement.
 
         Args:
@@ -458,8 +492,8 @@ class Database:
 
         return q
 
-    def insert(self, table, schema, select, columns=None, execute=False):
-        """Returns an INSERT statment from a SELECT query.
+    def _insert(self, table, schema, select, columns=None, execute=False):
+        """Returns an INSERT statement from a SELECT query.
 
         Args:
             table (str): The target table name
@@ -473,7 +507,7 @@ class Database:
         """
         table = f"{schema+'.' if schema else ''}{table}"
 
-        # we reshape the insert statement to avoid conflict if columns are not specified in same order between query and dag file
+        # we reshape the insert statement to avoid conflict if columns are not specified in same order between query and task group file
         if columns is not None:
             select = "SELECT i." + "\n, i.".join(columns) + f"\n\nFROM ({select}) AS i"
             columns = "(" + ", ".join(columns) + ")"
@@ -490,13 +524,13 @@ class Database:
 
         return q
 
-    def move_table(
+    def _move_table(
         self, src_table, src_schema, dst_table, dst_schema, ddl, execute=False
     ):
         """Returns SQL code to rename a table and change schema.
 
         Note:
-            Table movement is performed as a series of ALTER statments:
+            Table movement is performed as a series of ALTER statements:
 
               * ALTER TABLE RENAME
               * ALTER TABLE SET SCHEMA (if the database supports it)
@@ -520,28 +554,37 @@ class Database:
         else:
             change_schema = ""
 
+        pk_alter = []
+        if "NO ALTER INDEXES" not in self.sql_features and len(ddl["primary_key"]) > 0:
+            # Change primary key name
+            pk_alter.append(
+                f"ALTER INDEX {dst_schema+'.' if dst_schema else ''}{src_table}_pkey RENAME TO {dst_table}_pkey;"
+            )
+
         idx_alter = []
-        if ddl.get("indexes") is not None:
+        if len(ddl["indexes"]) > 0:
             # Change index names
             for idx in ddl["indexes"].keys():
-                if idx == "primary_key":
-                    # Primary keys are called as the table
+                if "NO ALTER INDEXES" in self.sql_features:
+                    idx_cols = " ,".join(ddl["indexes"][idx]["columns"])
                     idx_alter.append(
-                        f"ALTER INDEX {dst_schema+'.' if dst_schema else ''}{src_table}_pkey RENAME TO {dst_table}_pkey;"
+                        f"DROP INDEX {dst_schema+'.' if dst_schema else ''}{src_table}_{idx};\n"
+                        f"CREATE INDEX {dst_table}_{idx} ON {dst_table}({idx_cols});"
                     )
                 else:
                     idx_alter.append(
-                        f"ALTER INDEX {dst_schema+'.' if dst_schema else ''}{src_table}_{idx} RENAME TO {dst_table}_{idx};"
+                        f"ALTER INDEX {dst_schema+'.' if dst_schema else ''}{src_table}_{idx} "
+                        f"RENAME TO {dst_table}_{idx};"
                     )
 
-        q = "\n".join([rename, change_schema] + idx_alter)
+        q = "\n".join([rename, change_schema] + pk_alter + idx_alter)
 
         if execute:
             self.execute(q)
 
         return q
 
-    def merge_tables(
+    def _merge_tables(
         self,
         src_table,
         src_schema,
@@ -582,7 +625,7 @@ class Database:
         )
 
         select = f"SELECT * FROM {src}"
-        insert = self.insert(dst_table, dst_schema, select, columns=columns)
+        insert = self._insert(dst_table, dst_schema, select, columns=columns)
         q = "\n".join((delete, insert))
 
         if execute:
