@@ -6,6 +6,11 @@ from . import Task
 
 class BaseSqlTask(Task):
     target_table_exists = None
+    _target_db = None
+
+    @property
+    def target_db(self):
+        return self.connections[self._target_db]
 
     def run(self):
         return self.execute_steps()
@@ -16,17 +21,24 @@ class BaseSqlTask(Task):
     def execute_steps(self):
         # For incremental loads, manipulate the "Merge" steps depending on whether
         # the target table exists or not. This is done so we can delay the introspection
-        if "Merge" in self.steps and self.run_arguments["command"] == "run":
-            self.target_table_exists = self.default_db.table_exists(
+        if "Merge" in self.steps:
+            self.target_table_exists = self.target_db._table_exists(
                 self.table, self.schema
             )
 
             if not self.target_table_exists:
                 tmp = self.steps[self.steps.index("Merge") + 1 :]
                 self.steps = self.steps[: self.steps.index("Merge")]
-                if len(self.ddl["indexes"]) > 0:
+
+                cols_no_type = [c for c in self.ddl["columns"] if c["type"] is None]
+                if len(self.ddl["indexes"]) > 0 or (
+                    len(self.ddl["primary_key"]) > 0
+                    and len(self.ddl["columns"]) > 0
+                    and len(cols_no_type) > 0
+                ):
                     self.steps.append("Create Indexes")
                 self.steps.extend(["Cleanup Target", "Move"])
+
                 self.steps.extend(tmp)
             else:
                 self.steps.append("Cleanup")
@@ -41,13 +53,6 @@ class BaseSqlTask(Task):
 
         return Ok()
 
-    def create_table_ddl(self, tmp_table, tmp_schema, ddl, execute):
-        q = self.default_db.create_table_ddl(tmp_table, tmp_schema, ddl, execute)
-        if execute:
-            self.default_db.execute(q)
-
-        return Ok(q)
-
     def execute_step(self, step):
         execute = self.run_arguments["command"] == "run"
 
@@ -55,26 +60,26 @@ class BaseSqlTask(Task):
             "Create Temp": lambda: self.create_select(
                 self.tmp_table, self.tmp_schema, self.sql_query, self.ddl
             ),
-            "Create Temp DDL": lambda: self.default_db.create_table_ddl(
+            "Create Temp DDL": lambda: self.target_db._create_table_ddl(
                 self.tmp_table, self.tmp_schema, self.ddl
             ),
-            "Create View": lambda: self.default_db.create_table_select(
+            "Create View": lambda: self.target_db._create_table_select(
                 self.table, self.schema, self.sql_query, view=True
             ),
-            "Create Indexes": lambda: self.default_db.create_indexes(
+            "Create Indexes": lambda: self.create_indexes(
                 self.tmp_table, self.tmp_schema, self.ddl
             ),
-            "Merge": lambda: self.default_db.merge_tables(
+            "Merge": lambda: self.target_db._merge_tables(
                 self.tmp_table,
                 self.tmp_schema,
                 self.table,
                 self.schema,
                 self.delete_key,
             ),
-            "Move": lambda: self.default_db.move_table(
+            "Move": lambda: self.target_db._move_table(
                 self.tmp_table, self.tmp_schema, self.table, self.schema, self.ddl,
             ),
-            "Grant Permissions": lambda: self.default_db.grant_permissions(
+            "Grant Permissions": lambda: self.target_db.grant_permissions(
                 self.table, self.schema, self.ddl["permissions"]
             ),
         }
@@ -87,7 +92,7 @@ class BaseSqlTask(Task):
                 self.write_compilation_output(query, step.replace(" ", "_").lower())
             if execute:
                 try:
-                    self.default_db.execute(query)
+                    self.target_db.execute(query)
                 except Exception as e:
                     return Exc(e)
 
@@ -98,28 +103,44 @@ class BaseSqlTask(Task):
 
         elif step == "Execute Query":
             if execute:
-                self.default_db.execute(query)
+                try:
+                    self.target_db.execute(self.sql_query)
+                except Exception as e:
+                    return Exc(e)
 
             return Ok()
 
         elif step == "Cleanup":
-            return self.cleanup(self.tmp_table, self.tmp_schema, step, execute)
+            result = self.cleanup(self.tmp_table, self.tmp_schema, step, execute)
+            if result.is_err:
+                return result
+
+            return Ok()
 
         elif step == "Cleanup Target":
-            return self.cleanup(self.table, self.schema, step, execute)
+            result = self.cleanup(self.table, self.schema, step, execute)
+            if result.is_err:
+                return result
+
+            return Ok()
 
         elif step == "Load Data":
-            return self.load_data(
-                self.source_table_def,
-                self.source_db,
-                self.table,
-                self.schema,
-                self.tmp_table,
-                self.tmp_schema,
-                self.incremental_key,
-                self.ddl,
-                execute,
-            )
+            try:
+                self.load_data(
+                    self.source_table_def,
+                    self.source_db,
+                    self.table,
+                    self.schema,
+                    self.tmp_table,
+                    self.tmp_schema,
+                    self.incremental_key,
+                    self.ddl,
+                    execute,
+                )
+            except Exception as e:
+                return Exc(e)
+
+            return Ok()
 
         else:
             return Err("task_execution", "unknown_step", step=step)
@@ -129,45 +150,63 @@ class BaseSqlTask(Task):
     def create_select(self, table, schema, select, ddl):
         out_sql = list()
 
-        if len(ddl.get("columns")) == 0:
+        cols_no_type = [c for c in self.ddl["columns"] if c["type"] is None]
+        if len(ddl.get("columns")) == 0 or len(cols_no_type) > 0:
             out_sql.append(
-                self.default_db.create_table_select(
+                self.target_db._create_table_select(
                     table, schema, select, view=False, ddl=self.ddl
                 )
             )
         else:
             # create table with DDL and insert the output of the select
-            out_sql.append(self.default_db.create_table_ddl(table, schema, ddl))
+            out_sql.append(self.target_db._create_table_ddl(table, schema, ddl))
 
             ddl_column_names = [c["name"] for c in ddl.get("columns")]
             out_sql.append(
-                self.default_db.insert(table, schema, select, columns=ddl_column_names)
+                self.target_db._insert(table, schema, select, columns=ddl_column_names)
             )
 
         return "\n".join(out_sql)
 
+    def create_indexes(self, tmp_table, tmp_schema, ddl):
+        cols_no_type = [c for c in self.ddl["columns"] if c["type"] is None]
+        if not (len(ddl.get("columns")) == 0 or len(cols_no_type) > 0):
+            # Based on create_select: this condition means we're issuing a
+            # create_table_ddl, in which case we don't need an alter to
+            # add the primary key
+            ddl = {k: v if k != "primary_key" else dict() for k, v in ddl.items()}
+
+        return self.target_db._create_indexes(tmp_table, tmp_schema, ddl)
+
     def cleanup(self, table, schema, step, execute):
         out_sql = list()
 
-        try:
-            out_sql.append(
-                self.default_db.drop_table(table, schema, view=False, execute=execute)
-            )
-        except:
-            pass
+        # using those flags to capture error here. Not sure how to best capture a genuine error fail (e.g. permissions). To investigate.
+        cleanup_table_failed = False
+        cleanup_view_failed = False
 
         try:
             out_sql.append(
-                self.default_db.drop_table(table, schema, view=True, execute=execute)
+                self.target_db._drop_table(table, schema, view=False, execute=execute)
             )
         except:
-            pass
+            cleanup_table_failed = True
+
+        try:
+            out_sql.append(
+                self.target_db._drop_table(table, schema, view=True, execute=execute)
+            )
+        except:
+            cleanup_view_failed = True
 
         query = "\n".join(out_sql)
         if self.run_arguments["debug"]:
             self.write_compilation_output(query, step.replace(" ", "_").lower())
 
-        return Ok()
+        if cleanup_table_failed and cleanup_view_failed:
+            return Err("task_step", step, table=table, schema=schema)
+        else:
+            return Ok()
 
     def load_data(
         self,
@@ -198,7 +237,7 @@ class BaseSqlTask(Task):
 
         if not self.is_full_load and self.target_table_exists:
             if execute:
-                res = self.default_db.select(last_incremental_value_query)
+                res = self.target_db.read_data(last_incremental_value_query)
                 if len(res) == 1:
                     last_incremental_value = res[0]["value"]
             else:
@@ -216,7 +255,7 @@ class BaseSqlTask(Task):
             self.write_compilation_output(get_data_query, "get_data")
 
         if execute:
-            data_iter = source_db.select_stream(get_data_query)
-            return self.default_db.load_data(tmp_table, data_iter, schema=tmp_schema)
+            data_iter = source_db._read_data_stream(get_data_query)
+            return self.target_db.load_data(tmp_table, data_iter, schema=tmp_schema)
         else:
             return Ok()
